let sliderState = {
  action: '',
  slideIndex: 0,
  swipeOffset: 0,
  swipeDirection: null,
  swipeStatus: 'stop',
  length: 0,
};

let config = {
  swipeEndThreshold: 0.25,
  transitionTime: 400,
  numberOfSlides: 1,
  sliderGap: 0,
  style: {},
};

let conditions = {
  autoSlideWidth: false,
  slideWidth: 0,
  totalSlideWidth: 0,
  containerWidth: 0,
  containerPadding: 0,
  initIndex: 0,
};

// Debounce func used for swiping animation - to stop element jittering
export function debounceAnimationFrame(fn) {
  let frame;
  const debouncedFn = (...args) => {
    cancelAnimationFrame(frame);
    return new Promise((resolve) => {
      frame = requestAnimationFrame(() => {
        const result = fn(...args);
        resolve(result);
      });
    });
  };
  return debouncedFn;
}

function init(style, state, sliderContentRef, sliderContainerRef, settings, sliderConditionsRef) {
  config = { ...settings, style };
  conditions = { ...sliderConditionsRef.current };
  sliderState = state;

  let widthSize = `${(100 / config.numberOfSlides) * sliderState.length}%`;
  let colPercentage = `${100 / config.numberOfSlides / sliderState.length}%`;

  if (config.numberOfSlides === 0) {
    if (sliderContentRef.current === null) {
      widthSize = '100%';
      colPercentage = 'min-content';
    } else {
      widthSize = `${conditions.totalSlideWidth}px`;
      colPercentage = 'min-content';
    }
  }

  if (sliderContentRef.current !== null && conditions.slideWidth === 0) {
    const computedStyle = getComputedStyle(sliderContainerRef.current);
    conditions.containerPadding =
      parseFloat(computedStyle.paddingLeft) + parseFloat(computedStyle.paddingRight);

    conditions.containerWidth =
      sliderContainerRef.current.offsetWidth - conditions.containerPadding;
    conditions.slideWidth = sliderContentRef.current.offsetWidth / sliderState.length;

    if (config.numberOfSlides === 0) {
      conditions.autoSlideWidth = true;
      conditions.slideWidth = [];

      sliderContentRef.current.childNodes.forEach((child, index) => {
        let sliderGap = 0;

        if (index < sliderContentRef.current.childNodes.length - 1) {
          sliderGap = config.sliderGap;
        }
        conditions.slideWidth[index] = child.offsetWidth + sliderGap;
      });

      conditions.totalSlideWidth = conditions.slideWidth.reduce((a, b) => a + b, 0);
      widthSize = conditions.totalSlideWidth;
    }
  }

  config.style = {
    width: widthSize,
    gridTemplateColumns: `repeat(${sliderState.length}, ${colPercentage})`,
    gridGap: `${config.sliderGap}px`,
  };
}

// Slide for pagination and arrows
function navigate() {
  let translate = 0;

  // slide only within the right range
  if (
    Math.abs(sliderState.slideIndex) >= 0 &&
    Math.abs(sliderState.slideIndex) < sliderState.length
  ) {
    translate = conditions.slideWidth * sliderState.slideIndex;
  }

  config.style = {
    ...config.style,
    transition: `transform ${config.transitionTime}ms ease 0s`,
    transform: `translate3d(${translate + config.sliderGap * sliderState.slideIndex}px, 0px, 0px)`,
  };
}

// Swipe element with Slides
function swipeSlides() {
  let translate = 0;

  // Slide while swiping
  if (sliderState.swipeStatus === 'start') {
    conditions.initIndex = sliderState.slideIndex;
  }

  // While swiping
  if (
    sliderState.slideIndex === 0 &&
    sliderState.swipeDirection === 1 &&
    sliderState.swipeOffset >= conditions.slideWidth * config.swipeEndThreshold
  ) {
    // When on the first slide, sliding right
    translate = 0;
    config.style = {
      ...config.style,
      transition: `transform ${config.transitionTime}ms ease 0s`,
    };
  } else if (
    Math.abs(sliderState.slideIndex) === sliderState.length - 1 &&
    sliderState.swipeDirection === -1 &&
    sliderState.swipeOffset <= conditions.slideWidth * config.swipeEndThreshold * -1
  ) {
    // When on the last slide, sliding left
    sliderState.slideIndex = (sliderState.length - 1) * -1;
    translate = sliderState.slideIndex * conditions.slideWidth;
    config.style = {
      ...config.style,
      transition: `transform ${config.transitionTime}ms ease 0s`,
    };
  } else {
    // When sliding left and right, not on the first or last slide
    translate = sliderState.swipeOffset + conditions.initIndex * conditions.slideWidth;

    if (sliderState.swipeDirection === 1) {
      sliderState.slideIndex = Math.floor(translate / conditions.slideWidth);
    } else if (sliderState.swipeDirection === -1) {
      sliderState.slideIndex = Math.ceil(translate / conditions.slideWidth);
    }
  }

  // Before and after swipe
  if (sliderState.swipeStatus !== 'move') {
    // The factor is needed to find the next or previous slide position
    const factor = translate / conditions.slideWidth;

    // Fix the slide to snap in position once the swipe is complete
    if (sliderState.swipeDirection === 1) {
      const snapBackTo = Math.ceil(factor) * conditions.slideWidth;
      // Set a swipe treshhold to snap the slide back if the swipe was not long enough
      if (Math.abs(sliderState.swipeOffset) > conditions.slideWidth * config.swipeEndThreshold)
        translate = snapBackTo;
      else translate = Math.floor(factor) * conditions.slideWidth;
    } else if (sliderState.swipeDirection === -1) {
      const snapBackTo = Math.floor(factor) * conditions.slideWidth;
      // Set a swipe treshhold to snap the slide back if the swipe was not long enough
      if (Math.abs(sliderState.swipeOffset) > conditions.slideWidth * config.swipeEndThreshold)
        translate = snapBackTo;
      else translate = Math.ceil(factor) * conditions.slideWidth;
    }

    sliderState.slideIndex = translate / conditions.slideWidth;

    config.style = {
      ...config.style,
      transition: `transform ${config.transitionTime}ms ease 0s`,
    };
  }

  config.style = {
    ...config.style,
    transform: `translate3d(${translate + config.sliderGap * sliderState.slideIndex}px, 0px, 0px)`,
  };
}

// Swipe for element with not slides
function swipeRow() {
  let translate = 0;

  // If total slide width is bigger than the container, allow swiping

  if (conditions.totalSlideWidth > conditions.containerWidth) {
    // Slide while swiping
    if (
      sliderState.swipeDirection === -1 &&
      Math.abs(sliderState.slideIndex) - sliderState.swipeOffset >
        conditions.totalSlideWidth - conditions.containerWidth
    ) {
      translate = conditions.containerWidth - conditions.totalSlideWidth;
    } else if (
      sliderState.swipeDirection === 1 &&
      Math.abs(sliderState.slideIndex) - sliderState.swipeOffset < 0
    ) {
      translate = 0;
    } else {
      translate = sliderState.swipeOffset + sliderState.slideIndex;
    }

    // Before and after swipe
    if (sliderState.swipeStatus !== 'move') {
      sliderState.slideIndex = translate;
    }
  }

  config.style = {
    ...config.style,
    transform: `translate3d(${translate}px, 0px, 0px)`,
  };
}

// Slider Main
export function updateSlider(
  style,
  state,
  sliderContentRef,
  sliderContainerRef,
  settings,
  sliderConditionsRef
) {
  init(style, state, sliderContentRef, sliderContainerRef, settings, sliderConditionsRef);
  if (sliderContentRef.current !== null) {
    switch (sliderState.action) {
      case 'nav':
        navigate();
        break;
      case 'swipe':
        if (!conditions.autoSlideWidth) swipeSlides();
        else swipeRow();
        break;
      default:
        break;
    }
  }

  return [config.style, conditions];
}

// Swipe Controls

let swipeStatus = 'stop';
let swipeStartPoint = 0;

export function swipeStart(e, updateOnSwipeDebounce) {
  if (swipeStatus === 'stop') {
    swipeStatus = 'start';
    swipeStartPoint = e.touches && e.touches.length > 0 ? e.touches[0].clientX : e.clientX;
    updateOnSwipeDebounce(e, swipeStatus, swipeStartPoint);
  }
}

export function swipeMove(e, updateOnSwipeDebounce) {
  if (swipeStatus !== 'stop') {
    swipeStatus = 'move';
    updateOnSwipeDebounce(e, swipeStatus, swipeStartPoint);
  }
}

export function swipeStop(e, updateOnSwipeDebounce) {
  if (swipeStatus !== 'stop') {
    swipeStatus = 'stop';
    updateOnSwipeDebounce(e, swipeStatus, swipeStartPoint);
  }
}

export function conditionalClick(e) {
  if (swipeStatus !== 'start') {
    e.preventDefault();
    e.stopPropagation();
  }
}
